
ResistanceGAFunction <- function(id, par, n_cores, max_cont, max_cat,
                              location.file = sample.coords,
                              Gen.distance = genetic_distance,
                              ascii.dir = ascii.dir,
                              Version = NULL,
                              delete = TRUE) {
  set.seed(seed = id*0.125)
  library (ResistanceGA)
  library(rgdal)
  
  JULIA_HOME = 'C:/Users/User/AppData/Local/Julia-0.6.4/bin/'
  
  n_samples <- nrow(read.table(location.file))
  
  wd <- paste(temp.dir, par[id,"folder"], sep="")
  if(!dir.exists(wd)) dir.create(wd)
  setwd(wd)
  
  wd <- gsub("/*$", "/", wd)
  
  # Setting row index for which data1set to simulate
  stopifnot(id %in% par$id)
  ri <- which(par$id == id)[1]  # index of current run in par
  
  # Copy grids
  grids <- strsplit(as.character(par$files[ri]), split = " ")[[1]]  # split on space
  grids <- grids[nchar(grids) > 0]  # drop anything that doesn't have at least one character
  
  shapes <- strsplit(as.character(par$shape[ri]), split = " ")[[1]]  # split on space
  shapes <- shapes[nchar(shapes) > 0]  # drop anything that doesn't have at least one character
  
  trans <- strsplit(as.character(par$trans[ri]), split = " ")[[1]]  # split on space
  trans <- trans[nchar(trans) > 0]  # drop anything that doesn't have at least one character
  
  parms <- data.frame(grids=grids,shapes=NA,trans=NA)
  g_row <- 1
  for(g in grids){
    from <-  paste(ascii.dir, g, sep = "")
    to <- paste(wd, g, sep ="")
    file.copy(from, to)
    parms[g_row,"shapes"] <- shapes[g_row]
    parms[g_row,"trans"]<- trans[g_row]
    g_row <- g_row + 1
  }
  parms <- parms[order(parms$grids),]
  
  # Make a vector of cont.shape
  shapes <- parms$shapes
  
  # Make a list of allowable transformations
  trans <- as.list(parms$trans)
  
  for(i in 1:length(trans)){
    t1 <- trans[[i]]
    s1 <- shapes[i]
    if(!is.na(t1)&s1=="Decrease"&t1=="M"){
      trans[[i]] <- c(5,7)
    } else {
      if(!is.na(t1)&s1=="Increase"&t1=="M"){
        trans[[i]] <- c(3,1)
      } else {
        if(!is.na(t1)&t1=="A"){
          trans[[i]] <- c(1,3,5,7,8,2)
        } else {
          if(!is.na(t1)&s1=="Increase"&t1=="1"){
            trans[[i]] <- c(1)
          } else {
            if(t1=="M"&&s1=="NULL"){
              trans[[i]] <- c(1,3,5,7)
            } else {
              if(t1=="NULL"&&s1=="NULL"){
                trans[[i]] <- "NULL"
              } else {
                stop("Check constraint on functional transformation")
              }
            }
          }
        }
      }
    }
  }
  
  if(sum(grepl("NA",trans))>0) trans[grepl("NA",trans)] <- NA
  parms$scale <- 1
  if(sum(grepl("water",as.character(parms$grids),ignore.case=T))>0) parms[grepl("water",
                                                                                parms$grids,
                                                                                ignore.case=T),"scale"] <- 0
  
  # Call function
  shapes <- "NULL"
  #ResistanceGA
  if(par[ri,"method"]!="Julia"){
    GA.inputs <- GA.prep(ASCII.dir=wd,
                         Results.dir=wd,
                         cont.shape=shapes,
                         select.trans = trans,
                         # max.scale = par[id,"max_scale"]/cell_size,
                         max.cat=max_cat,
                         max.cont=max_cont,
                         parallel = n_cores,
                         # quiet = FALSE,
                         maxiter=1000,
                         pop.mult = 25,
                         run = 45
                         # scale=TRUE,
                         # scale.surfaces = parms$scale
    )
  } else {
    print("Setting up GA.prep for using JULIA")
    
    locales <- read.delim(location.file, header = F) # Read in location file
    samp.pts <- SpatialPoints(locales[,2:3])
    
    jl.inputs <- jl.prep(n.Pops = length(samp.pts),
                         response = Gen.distance,
                         CS_Point.File = samp.pts,
                         cholmod = T,
                         JULIA_HOME = JULIA_HOME#,
                         # parallel = TRUE,
                         # cores = 2
    )
    
    GA.inputs <- GA.prep(ASCII.dir=wd,
                         Results.dir=wd,
                         cont.shape=shapes,
                         select.trans = trans,
                         # max.scale = par[id,"max_scale"]/cell_size,
                         max.cat=max_cat,
                         max.cont=max_cont,
                         parallel = n_cores,
                         gaisl = FALSE,
                         # island.pop = 25,
                         # numIslands = 10,
                         run = 50,
                         maxiter = 1000,
                         # quiet = FALSE,
                         # maxiter=1000
                         pop.mult = 30
                         # scale=TRUE,
                         # scale.surfaces = parms$scale
    )
  }
  
  if(par[ri,"method"]=="Julia"){
    
    if(length(grids)>1){
      start_time <- Sys.time()
      system.time(SS_RESULTS <- MS_optim(jl.inputs = jl.inputs,
                           GA.inputs = GA.inputs))
      end_time <- Sys.time()
    } else {
      start_time <- Sys.time()
      system.time(SS_RESULTS <- SS_optim(jl.inputs = jl.inputs,
                              GA.inputs = GA.inputs))
      end_time <- Sys.time()
    }
    
  } else {
    gdist.inputs <- gdist.prep(n.Pops = n_samples,
                               response = Gen.distance,
                               samples=location.file,
                               method=par[ri,"method"])
    
    if(length(grids)>1){
      if(!is.null(GA.inputs$scale)){
        stop("Should not be running")
        system.time(SS_RESULTS <- MS_optim.scale(gdist.inputs = gdist.inputs,
                                                 GA.inputs = GA.inputs))
      } else {
        start_time <- Sys.time()
        system.time(SS_RESULTS <- MS_optim(gdist.inputs = gdist.inputs,
                                           GA.inputs = GA.inputs))
        end_time <- Sys.time()
      }
    } else {
      if(!is.null(GA.inputs$scale)){
        stop("Should not be running")
        system.time(SS_RESULTS <- SS_optim.scale(gdist.inputs = gdist.inputs,
                                                 GA.inputs = GA.inputs))
      } else {
        start_time <- Sys.time()
        system.time(SS_RESULTS <- SS_optim(gdist.inputs = gdist.inputs,
                                           GA.inputs = GA.inputs))
        end_time <- Sys.time()
      }
    }
  }
  total_time <- end_time-start_time
  # Save results
  res.file <- paste(wd, par[id,"folder"], ".Rdata", sep = "")
  save(SS_RESULTS, id, par, file = res.file)
  out_list <- list("GA.inputs"=GA.inputs,
                   "par"=par,
                   "id"=id,
                   "n_cores"=n_cores,
                   "version"=Version,
                   "computer"="Desktop",
                   "total_time"=total_time)
  saveRDS(out_list,paste(wd, par[id,"folder"], ".rds", sep = ""))
  
  # Delete output
  if(delete){
    setwd(temp.dir)
    unlink(wd, recursive = TRUE)  
  }
  
  
}  # end function

